# Simulatore di Server Modbus

### Introduzione

Questo progetto realizza un server modbus che sia in grado di simulare una macchina fisica la quale utilizza questo protocollo di comunicazione per scambiare dati applicativi.

Requisito del simulatore è il poter rendere disponibili alla lettura N registri di ognuno dei tipi definiti nel protocollo modbus (Discrete Output, Discrete Input, Holding Registers, Input Registers) inizializzati con un valore di default. Ogni registro è aggiornabile ad un valore arbitrario, effettuando operazioni di scrittura tramite un client.

**Unicamente** per quanto riguarda le tipologie di **registri "Holding Registers"**, è implementata una soluzione di **update automatico** dei valori, allo scopo di meglio simulare le variazioni di funzionamento di una macchina fisica.

### Struttura del progetto

Il progetto ( che utilizza il linguaggio **python 3** ) è strutturato secondo packages, ognuno dei quali raggruppa un ambito funzionale. Le operazioni di build, test e creazione del package, sono automatizzate utilizzando lo strumento built-in *setuptools*.

Il progetto è composto dei seguenti package:

- **modbus:**  esso contiene il modulo che implementa il server ed anche il modulo client che effettua gli update programmati per la tipologia di registri Holding Registers
- **tests:** esso contiene tutti i test unitari e di integrazione
- **utility:** questo contiene alcuni moduli di configurazione (logger, costanti) , un modulo che integra una classe in grado di creare un thread schedulando la ripetizione - ad intervalli definiti - di una specifica funzione (in questo caso la funzione è l'update automatico dei registri sopra indicati), un modulo generatore di una forma d'onda (dente di sega) in grado di fornire un pattern riconoscibile per (se previsto) l'aggiornamento automatico dei registri.
- **hmi_server:** questo package contiene un unico modulo, il quale rappresenta l'entry point dell'applicazione, istanziando sia il server modbus, sia gli eventuali job di update automatico dei registri Holding Registers  

E' presente inoltre un file di configurazione *config.yaml*, il quale permette di configurare in toto il funzionamento dell'applicazione.

### Configurazione

Il file *config.yaml* rappresenta l'unico punto di  intervento per la modifica del funzionamento dell'applicazione; esso è composto da:

1) configurazioni relative al modbus

```yaml
modbus:
  server : 'localhost'
  port : 2502
  listen_interface : "0.0.0.0"
  slave_unit : 0x00
  total_di_registers: 10
  total_co_registers: 10
  total_hr_registers: 3010
  total_ir_registers: 10
```

Questo blocco di codice - **mandatorio** - identifica i vari parametri di funzionamento del server modbus.
Le properties **server** e **slave_unit** vengono utilizzate dal client built-in che effettua le operazioni di update dei registri; esse non dovrebbero mai essere modificate.

**listen_interface** indica l'interfaccia su cui il server è in ascolto, mentre **port** indica la porta su cui è esposto.

Infine, in corrispondenza dei vari registri, viene indicato un intero che rappresenta il numero totale di registri (dello specifico tipo) che si desidera inizializzare .

Sotto vengono riportati ulteriori properties - **mandatorie** - , le quali sono a completa discrezione utente, e rappresentano i metadati di identificazione del server.

```yaml
modbus_server_info:
  vendor_name : 'ImolaInformatica'
  product_code : 'XYZ145'
  vendor_url : 'https://www.imolainformatica.it'
  product_name : 'HMIsimulator'
  model_name : 'HMIsimulator'
  revision : '0.1'
```

2) Configurazioni relative ai job di update dei registri "Holding Registers"

Di seguito viene riportato un esempio di definizione di alcuni job di update. Si noti che l'intero blocco seguente **è opzionale** e può mancare in toto. E' definibile un numero arbitrario di jobs.

```yaml
updater_jobs:
  tempi_attuatori:
    start_register : 1
    stop_register : 29
    frequency : 5
  temperature:
    start_register : 1000
    stop_register : 1027
    frequency : 10
  temperature_stampo:
    start_register: 2000
    stop_register: 2071
    frequency: 10
  temperature_acqua:
    start_register: 3000
    stop_register: 3006
    frequency: 10
```

Di seguito un estratto che riporta la struttura dati - **mandatoria** - che permette di definire un job di update:

```yaml
tempi_attuatori:
    start_register : 1
    stop_register : 29
    frequency : 5
```

In sostanza viene definito quale sarà l'**intervallo di registr**i da aggiornare (estremi compresi) e la **frequenza** con la quale avverrà la strategia di aggiornamento.

Il nome del job (es: tempi_attuatori) è a discrezione dell'utente.

### Note importanti

Si notino alcune caratteristiche importanti:

- Per ogni job di update schedulato verrà creato un thread.
- L'aggiornamento automatico dei registri è previsto solo per la tipologia Holding Register, e si avvale del modulo di generazione di forma d'onda allo scopo di rappresentare, nel tempo, un pattern riconoscibile (dente di sega) che renda gradevole la visualizzazione dei dati.
- E' ovviamente possibile utilizzare lo strumento come server modbus puro, eliminando completamente il blocco di configurazione "**updater_jobs**",  andando a definire esternamente una eventuale politica di aggiornamento dei dati dei registri.  

### Installazione, test

Per effettuare i test eseguire (posizionandosi nella root del progetto):

```shell
python3 setup.py test
```

Per effettuare una verifica della coverage è necessario installare prima il pacchetto:

```
pip3 install coverage
```

e poi eseguire il comando;

```
coverage3 run setup.py test
```

è quindi possibile ottenere un report con - in alternativa - uno dei seguenti comandi

```shell
# report a terminale
coverage3 report
# conversione per lettura da sonar
coverage3 xml
# report grafico
coverage3 html

```

Per generare il pacchetto di installazione è possibile utilizzare il comando:

```
python3 setup.py bdist_wheel
```

In questo modo verrà generato un file .whl che è direttamente installabile utilizzando pip. Tutte le dipendenze necessarie verranno soddisfatte automaticamente.

In alternativa è possibile un'installazione direttamente dai sorgenti, utilizzando il comando:

```
python3 setup.py install
```

### Utilizzo

E' possibile utilizzare l'applicazione in due modi:

1. **In seguito all'installazione:** in questo caso verrà creato un alias che permette di eseguire l'applicazione direttamente dalla shell. dovrà essere fornito come argomento il percorso verso il file di configurazione.

   ```shell
   hmi-simulator /path/to/folder/config.yaml
   ```

2. **Direttamente dai sorgenti:** in questo caso nessuna installazione è richiesta, ma basterà posizionarsi nella root del progetto ed utilizzare il comando:

   ```
   python3 -m hmi_server.hmi_server /path/to/folder/config.yaml
   ```