from pymodbus.server.sync import StartTcpServer
from pymodbus.device import ModbusDeviceIdentification
from pymodbus.datastore import ModbusSequentialDataBlock
from pymodbus.datastore import ModbusSlaveContext, ModbusServerContext
from hmi_simulator.utility.properties_constants import *
from hmi_simulator.utility.logger import *


def run_server(properties):
    # get parameters from properties
    modbus_listen_interface = properties[MODBUS][LISTEN_INTERFACE]
    modbus_port = properties[MODBUS][PORT]
    total_hr_registers = properties[MODBUS][TOTAL_HR_REGISTERS]
    total_di_registers = properties[MODBUS][TOTAL_DI_REGISTERS]
    total_co_registers = properties[MODBUS][TOTAL_CO_REGISTERS]
    total_ir_registers = properties[MODBUS][TOTAL_IR_REGISTERS]
    vendor_name = properties[MODBUS_SERVER_INFO][VENDOR_NAME]
    product_code = properties[MODBUS_SERVER_INFO][PRODUCT_CODE]
    vendor_url = properties[MODBUS_SERVER_INFO][VENDOR_URL]
    product_name = properties[MODBUS_SERVER_INFO][PRODUCT_NAME]
    model_name = properties[MODBUS_SERVER_INFO][MODEL_NAME]
    revision = properties[MODBUS_SERVER_INFO][REVISION]

    try:
        # initialize data store: (start register, [default value to be written]* number of registers)
        store = ModbusSlaveContext(
            di=ModbusSequentialDataBlock(0, [1]*total_di_registers),
            co=ModbusSequentialDataBlock(0, [2]*total_co_registers),
            hr=ModbusSequentialDataBlock(0, [3]*total_hr_registers),
            ir=ModbusSequentialDataBlock(0, [4]*total_ir_registers))

        context = ModbusServerContext(slaves=store, single=True)

        # initialize the server information
        identity = ModbusDeviceIdentification()
        identity.VendorName = vendor_name
        identity.ProductCode = product_code
        identity.VendorUrl = vendor_url
        identity.ProductName = product_name
        identity.ModelName = model_name
        identity.MajorMinorRevision = revision

        # start server
        log.info("Starting Server; listening interface: {} port: {}".format(modbus_listen_interface, modbus_port))
        StartTcpServer(context, identity=identity, address=(modbus_listen_interface, modbus_port))

    except Exception as e:
        log.debug(e)
        raise
