from pymodbus.client.sync import ModbusTcpClient as ModbusClient
from hmi_simulator.utility.func_generator import saw as saw_generator
from hmi_simulator.utility.properties_constants import *
from hmi_simulator.utility.logger import *


def update_holding_registers(job_name, properties):
    # get parameters from properties
    modbus_server_port = properties[MODBUS][PORT]
    modbus_server_addr = properties[MODBUS][SERVER]
    modbus_slave_unit = properties[MODBUS][SLAVE_UNIT]
    start_register = properties[UPDATER_JOBS][job_name][START_REGISTER]
    stop_register = properties[UPDATER_JOBS][job_name][STOP_REGISTER]

    try:
        client = ModbusClient(modbus_server_addr, port=modbus_server_port)
        client.connect()

        # because of the implementation of range() function, we need to increment stop register by 1 in order to have intuitive range in function call
        # otherwise setting range from register 100 to 110 would result in access limited to 100 to 109 registers

        # read value form each register and update that value
        for register in range(start_register, stop_register + 1):

            # read holding register value.
            # Since this function allows multiple register reading, args require two int:
            # 1) start register position and 2) how many registers need to be read.

            register_value = client.read_holding_registers(register, 1, unit=modbus_slave_unit)
            # modify register value to represent saw wave
            updated_register_value = saw_generator(SAW_GEN_AMPLITUDE, SAW_MIN_VAL, register_value.registers[0])
            # write new value
            rq = client.write_register(register, updated_register_value, unit=modbus_slave_unit)

            assert(not rq.isError())
            log.info("Value of register {} updated to: {}".format(register, updated_register_value))

        log.info("Holding registers from position {} to {} updated".format(start_register, stop_register))

    except Exception as e:
        log.info(e)
        raise

    finally:
        client.close()
