import os
import yaml
from sys import argv
from functools import partial
from hmi_simulator.modbus import updater as modbus_updater, server as modbus_server
from hmi_simulator.scheduler import multithread_scheduler as scheduler
from hmi_simulator.utility.properties_constants import *
from hmi_simulator.utility.logger import *


def load_config(configfile):

    if not os.path.isfile(configfile):
        raise Exception("Config file specified as arguments is incorrect or non existent")
    else:
        try:
            log.info("Importing properties from config file")
            with open(configfile, 'r') as file:
                properties = yaml.load(file)
        except Exception as e:
            log.debug(e)
            raise

    return properties


def main():
    try:
        # get arguments
        configfile = argv[1]
        # load properties from file
        properties = load_config(configfile)

        # check if there are updating jobs to be scheduled
        if UPDATER_JOBS in properties:
            # read all jobs that needs to be scheduled
            update_jobs_list = properties[UPDATER_JOBS]

            for update_job in update_jobs_list:
                # get schedule update from properties
                frequency = properties[UPDATER_JOBS][update_job][FREQUENCY]
                # update registers in background.
                scheduler.MultiThreadScheduler(partial(modbus_updater.update_holding_registers, update_job, properties), int(frequency))
                log.info("Updating job: {} scheduled".format(str(update_job)))
        else:
            log.info("No update jobs found in configuration file, scheduler skipped")

        # run modbus server
        modbus_server.run_server(properties)

    except Exception as e:
        log.info(e)
        raise


if __name__ == "__main__":
    main()

